@@ -9,9 +9,9 @@ arch_get_kernel_flavour () {
 			;;
 	esac
 	case "$SUBARCH" in
-		powermac*|prep|chrp*)	echo "$family" ;;
-		amiga)			echo apus ;;
-		ps3)			echo ps3 ;;
+		powermac*|prep|chrp*)		echo "$family" ;;
+		ps3|cell)			echo powerpc64 ;;
+		amiga)				echo apus ;;
 		*)
 			warning "Unknown $ARCH subarchitecture '$SUBARCH'."
 			return 1
@@ -47,12 +47,7 @@ arch_get_kernel () {
 			echo "linux-$1"
 			echo "linux-image-$1"
 			;;
-		*)
-			if [ "$1" = powerpc ] && [ "$SMP" ]; then
-				# 2.4 only has powerpc-smp.
-				echo "kernel-image-$KERNEL_MAJOR-$1$SMP"
-			fi
-			echo "kernel-image-$KERNEL_MAJOR-$1"
+		*)      warning "Unsupported kernel major '$KERNEL_MAJOR'."
 			;;
 	esac
 }
