@@ -652,6 +652,9 @@ get_resume_partition () {
 		if [ "$type" != partition ]; then
 			continue
 		fi
+		if [ "${filename#/dev/ramzswap}" != "$filename" ]; then
+			continue
+		fi
 		if [ "$size" -gt "$biggest_size" ]; then
 			biggest_size="$size"
 			biggest_partition="$filename"
@@ -772,7 +775,7 @@ EOT
 		else
 			SECMIRROR="$MIRROR"
 		fi
-		echo "deb $PROTOCOL://$SECMIRROR/ubuntu $DISTRIBUTION-security $COMPONENTS" >> /target/etc/apt/sources.list
+		echo "deb $PROTOCOL://$MIRROR/ubuntu $DISTRIBUTION-security $COMPONENTS" >> /target/etc/apt/sources.list
 		if db_get apt-setup/proposed && [ "$RET" = true ]; then
 			echo "deb $APTSOURCE $DISTRIBUTION-proposed $COMPONENTS" >> /target/etc/apt/sources.list
 		fi
