@@ -15,7 +15,6 @@
 #endif
 
 #include "partconf.h"
-#include <debian-installer.h>
 
 // If it's an LVM volume, it's on the form
 // /dev/<group>/<volume> and there's info in
@@ -204,18 +203,7 @@ get_all_partitions(struct partition *parts[], const int max_parts, bool ignore_f
 
             p = malloc(sizeof(*p));
             p->path = ped_partition_get_path(part);
-            if (strstr(p->path, "/dev/ide/") == p->path) {
-                static char *targets[] = { "master", "slave" };
-                int host, bus, target, lun, part;
-
-                if (sscanf(p->path, "/dev/ide/host%d/bus%d/target%d/lun%d/part%d",
-                            &host, &bus, &target, &lun, &part) == 5
-                        && target >= 0 && target <= 1)
-                    asprintf(&p->description, "IDE%d %s\\, part. %d",
-                            2*host + bus + 1, targets[target], part);
-                else
-                    p->description = strdup(p->path);
-            } else if (strstr(p->path, "/dev/hd") == p->path) {
+            if (strstr(p->path, "/dev/hd") == p->path) {
                 static char *targets[] = { "master", "slave" };
                 char drive;
                 int part;
@@ -227,15 +215,6 @@ get_all_partitions(struct partition *parts[], const int max_parts, bool ignore_f
                             part);
                 else
                     p->description = strdup(p->path);
-            } else if (strstr(p->path, "/dev/scsi/") == p->path) {
-                int host, bus, target, lun, part;
-
-                if (sscanf(p->path, "/dev/scsi/host%d/bus%d/target%d/lun%d/part%d",
-                            &host, &bus, &target, &lun, &part) == 5)
-                    asprintf(&p->description, "SCSI%d (%d\\,%d\\,%d) part. %d",
-                            host + 1, bus, target, lun, part);
-                else
-                    p->description = strdup(p->path);
             } else if (strstr(p->path, "/dev/sd") == p->path) {
                 char drive;
                 int host, bus, target, lun, part;
