@@ -36,6 +36,8 @@ import traceback
 import syslog
 import gzip
 import debconf
+import warnings
+warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
 import apt_pkg
 from apt.cache import Cache
 from apt.progress import FetchProgress, InstallProgress
@@ -281,6 +283,13 @@ class Install:
         self.kernel_version = platform.release()
         self.db = debconf.Debconf()
 
+        self.select_language_packs()
+        self.blacklist = {}
+        if self.db.get('ubiquity/install/generate-blacklist') == 'true':
+            self.db.progress('START', 0, 100, 'ubiquity/install/title')
+            self.db.progress('INFO', 'ubiquity/install/blacklist')
+            self.generate_blacklist()
+
         apt_pkg.InitConfig()
         apt_pkg.Config.Set("Dir", "/target")
         apt_pkg.Config.Set("Dir::State::status", "/target/var/lib/dpkg/status")
@@ -428,6 +437,8 @@ class Install:
             self.db.progress('INFO', 'ubiquity/install/removing')
             self.remove_extras()
 
+            self.remove_broken_cdrom()
+
             self.db.progress('SET', 99)
             self.db.progress('INFO', 'ubiquity/install/log_files')
             self.copy_logs()
@@ -443,6 +454,165 @@ class Install:
                 pass
 
 
+    def copy_file(self, sourcepath, targetpath, md5_check):
+        sourcefh = None
+        targetfh = None
+        try:
+            while 1:
+                sourcefh = open(sourcepath, 'rb')
+                targetfh = open(targetpath, 'wb')
+                if md5_check:
+                    sourcehash = md5()
+                while 1:
+                    buf = sourcefh.read(16 * 1024)
+                    if not buf:
+                        break
+                    targetfh.write(buf)
+                    if md5_check:
+                        sourcehash.update(buf)
+
+                if not md5_check:
+                    break
+                targetfh.close()
+                targetfh = open(targetpath, 'rb')
+                if md5_check:
+                    targethash = md5()
+                while 1:
+                    buf = targetfh.read(16 * 1024)
+                    if not buf:
+                        break
+                    targethash.update(buf)
+                if targethash.digest() != sourcehash.digest():
+                    if targetfh:
+                        targetfh.close()
+                    if sourcefh:
+                        sourcefh.close()
+                    error_template = 'ubiquity/install/copying_error/md5'
+                    self.db.subst(error_template, 'FILE', targetpath)
+                    self.db.input('critical', error_template)
+                    self.db.go()
+                    response = self.db.get(error_template)
+                    if response == 'skip':
+                        break
+                    elif response == 'abort':
+                        syslog.syslog(syslog.LOG_ERR,
+                            'MD5 failure on %s' % targetpath)
+                        sys.exit(3)
+                    elif response == 'retry':
+                        pass
+                else:
+                    break
+        finally:
+            if targetfh:
+                targetfh.close()
+            if sourcefh:
+                sourcefh.close()
+
+    def find_cd_kernel(self):
+        """Find the boot kernel on the CD, if possible."""
+
+        release_bits = os.uname()[2].split('-')
+        if len(release_bits) >= 3:
+            subarch = release_bits[2]
+        else:
+            subarch = None
+
+        for prefix in ('vmlinux', 'vmlinuz'):
+            kernel = os.path.join('/cdrom/casper', prefix)
+            if os.path.exists(kernel):
+                return kernel
+
+            if subarch:
+                kernel = os.path.join('/cdrom/casper', subarch, prefix)
+                if os.path.exists(kernel):
+                    return kernel
+
+                kernel = os.path.join('/cdrom/casper',
+                                      '%s-%s' % (prefix, subarch))
+                if os.path.exists(kernel):
+                    return kernel
+
+        return None
+
+    def generate_blacklist(self):
+        if (os.path.exists("/cdrom/casper/filesystem.manifest-desktop") and
+            os.path.exists("/cdrom/casper/filesystem.manifest")):
+            desktop_packages = set()
+            manifest = open("/cdrom/casper/filesystem.manifest-desktop")
+            for line in manifest:
+                if line.strip() != '' and not line.startswith('#'):
+                    desktop_packages.add(line.split()[0])
+            manifest.close()
+            live_packages = set()
+            manifest = open("/cdrom/casper/filesystem.manifest")
+            for line in manifest:
+                if line.strip() != '' and not line.startswith('#'):
+                    live_packages.add(line.split()[0])
+            manifest.close()
+            difference = live_packages - desktop_packages
+        else:
+            difference = set()
+        
+        # Keep packages we explicitly installed.
+        difference -= self.query_recorded_installed()
+        archdetect = subprocess.Popen(['archdetect'], stdout=subprocess.PIPE)
+        subarch = archdetect.communicate()[0].strip()
+
+        # Less than ideal.  Since we cannot know which bootloader we'll need
+        # at file copy time, we should figure out why grub still fails when
+        # apt-install-direct is present during configure_bootloader (code
+        # removed).
+        if subarch.startswith('amd64/') or subarch.startswith('i386/') or subarch.startswith('lpia/'):
+            difference -= set(['grub'])
+        elif subarch == 'powerpc/ps3':
+            pass
+        elif subarch.startswith('powerpc/'):
+            difference -= set(['yaboot', 'hfsutils'])
+ 
+        if len(difference) == 0:
+            self.blacklist = {}
+            return
+ 
+        use_restricted = True
+        try:
+            if self.db.get('apt-setup/restricted') == 'false':
+                use_restricted = False
+        except debconf.DebconfError:
+            pass
+        if not use_restricted:
+            cache = Cache()
+            for pkg in cache.keys():
+                if (cache[pkg].isInstalled and
+                    cache[pkg].section.startswith('restricted/')):
+                    difference.add(pkg)
+            del cache
+        cache = Cache()
+        confirmed_remove = set()
+        for pkg in sorted(difference):
+            if pkg in confirmed_remove:
+                continue
+            would_remove = self.get_remove_list(cache, [pkg], recursive=True)
+            if would_remove <= difference:
+                confirmed_remove |= would_remove
+                # Leave these marked for removal in the apt cache to speed
+                # up further calculations.
+            else:
+                for removedpkg in would_remove:
+                    cachedpkg = self.get_cache_pkg(cache, removedpkg)
+                    cachedpkg.markKeep()
+        difference = confirmed_remove
+        difference = set(filter(
+            lambda x: not os.path.exists('/var/lib/dpkg/info/%s.prerm' % x),
+            difference))
+        cmd = ['dpkg', '-L']
+        cmd.extend(difference)
+        subp = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        res = subp.communicate()[0].splitlines()
+        u = {}
+        for x in res:
+            u[x] = 1
+        self.blacklist = u
+
     def copy_all(self):
         """Core copy process. This is the most important step of this
         stage. It clones live filesystem into a local partition in the
@@ -473,10 +643,12 @@ class Install:
 
             for name in dirnames + filenames:
                 relpath = os.path.join(sourcepath, name)
-                fqpath = os.path.join(self.source, dirpath, name)
-
-                total_size += os.lstat(fqpath).st_size
-                files.append(relpath)
+                fqpath = os.path.join(dirpath, name)
+                # /etc/fstab was legitimately created by partman, and
+                # shouldn't be copied again.
+                if relpath != "etc/fstab":
+                    total_size += os.lstat(fqpath).st_size
+                    files.append(relpath)
 
         self.db.progress('SET', 10)
         self.db.progress('INFO', 'ubiquity/install/copying')
@@ -508,9 +680,10 @@ class Install:
             st = os.lstat(sourcepath)
             mode = stat.S_IMODE(st.st_mode)
             if stat.S_ISLNK(st.st_mode):
-                if not os.path.lexists(targetpath):
-                    linkto = os.readlink(sourcepath)
-                    os.symlink(linkto, targetpath)
+                if os.path.lexists(targetpath):
+                    os.unlink(targetpath)
+                linkto = os.readlink(sourcepath)
+                os.symlink(linkto, targetpath)
             elif stat.S_ISDIR(st.st_mode):
                 if not os.path.isdir(targetpath):
                     os.mkdir(targetpath, mode)
@@ -523,59 +696,12 @@ class Install:
             elif stat.S_ISSOCK(st.st_mode):
                 os.mknod(targetpath, stat.S_IFSOCK | mode)
             elif stat.S_ISREG(st.st_mode):
-                if not os.path.exists(targetpath):
-                    sourcefh = None
-                    targetfh = None
-                    try:
-                        while 1:
-                            sourcefh = open(sourcepath, 'rb')
-                            targetfh = open(targetpath, 'wb')
-                            if md5_check:
-                                sourcehash = md5()
-                            while 1:
-                                buf = sourcefh.read(16 * 1024)
-                                if not buf:
-                                    break
-                                targetfh.write(buf)
-                                if md5_check:
-                                    sourcehash.update(buf)
-
-                            if not md5_check:
-                                break
-                            targetfh.close()
-                            targetfh = open(targetpath, 'rb')
-                            if md5_check:
-                                targethash = md5()
-                            while 1:
-                                buf = targetfh.read(16 * 1024)
-                                if not buf:
-                                    break
-                                targethash.update(buf)
-                            if targethash.digest() != sourcehash.digest():
-                                if targetfh:
-                                    targetfh.close()
-                                if sourcefh:
-                                    sourcefh.close()
-                                error_template = 'ubiquity/install/copying_error/md5'
-                                self.db.subst(error_template, 'FILE', targetpath)
-                                self.db.input('critical', error_template)
-                                self.db.go()
-                                response = self.db.get(error_template)
-                                if response == 'skip':
-                                    break
-                                elif response == 'abort':
-                                    syslog.syslog(syslog.LOG_ERR,
-                                        'MD5 failure on %s' % targetpath)
-                                    sys.exit(3)
-                                elif response == 'retry':
-                                    pass
-                            else:
-                                break
-                    finally:
-                        if targetfh:
-                            targetfh.close()
-                        if sourcefh:
-                            sourcefh.close()
+                if '/%s' % path in self.blacklist:
+                    syslog.syslog('Not copying %s' % path)
+                    continue
+                if os.path.exists(targetpath):
+                    os.unlink(targetpath)
+                self.copy_file(sourcepath, targetpath, md5_check)
 
             copied_size += st.st_size
             os.lchown(targetpath, st.st_uid, st.st_gid)
@@ -620,6 +746,22 @@ class Install:
                 # about this failing, but I really don't care. Ignore it.
                 pass
 
+        # Try some possible locations for the kernel we used to boot. This
+        # lets us save a couple of megabytes of CD space.
+        bootdir = os.path.join(self.target, 'boot')
+        kernel = self.find_cd_kernel()
+        if kernel:
+            prefix = os.path.basename(kernel).split('-', 1)[0]
+            release = os.uname()[2]
+            target_kernel = os.path.join(bootdir, '%s-%s' % (prefix, release))
+            if os.path.exists(target_kernel):
+                os.unlink(target_kernel)
+            self.copy_file(kernel, target_kernel, md5_check)
+            os.lchown(target_kernel, 0, 0)
+            os.chmod(target_kernel, 0644)
+            st = os.lstat(kernel)
+            os.utime(target_kernel, (st.st_atime, st.st_mtime))
+
         os.umask(old_umask)
 
         self.db.progress('SET', 100)
@@ -637,10 +779,11 @@ class Install:
                          '/var/log/installer/version', '/var/log/casper.log'):
             target_log_file = os.path.join(target_dir,
                                            os.path.basename(log_file))
-            if not misc.execute('cp', '-a', log_file, target_log_file):
-                syslog.syslog(syslog.LOG_ERR,
-                              'Failed to copy installation log file')
-            os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
+            if os.path.isfile(log_file):
+                if not misc.execute('cp', '-a', log_file, target_log_file):
+                    syslog.syslog(syslog.LOG_ERR,
+                                  'Failed to copy installation log file')
+                os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
         try:
             status = open(os.path.join(self.target, 'var/lib/dpkg/status'))
             status_gz = gzip.open(os.path.join(target_dir,
@@ -710,7 +853,10 @@ class Install:
             mountpoint = '/var/lib/ubiquity/%s' % sysloop
         if not os.path.isdir(mountpoint):
             os.mkdir(mountpoint)
-        misc.execute('mount', dev, mountpoint)
+        if not misc.execute('mount', dev, mountpoint):
+            misc.execute('losetup', '-d', dev)
+            misc.execute('mount', '-o', 'loop', fsfile, mountpoint)
+            dev = 'unused'
 
         return (dev, mountpoint)
 
@@ -741,7 +887,8 @@ class Install:
             # Manual detection on non-unionfs systems
             fsfiles = ['/cdrom/casper/filesystem.cloop',
                        '/cdrom/casper/filesystem.squashfs',
-                       '/cdrom/META/META.squashfs']
+                       '/cdrom/META/META.squashfs',
+                       '/live/image/live/filesystem.squashfs']
 
             for fsfile in fsfiles:
                 if fsfile != '' and os.path.isfile(fsfile):
@@ -791,7 +938,8 @@ class Install:
             if not misc.execute('umount', mountpoint):
                 raise InstallStepError("Failed to unmount %s" % mountpoint)
         for dev in devs:
-            if dev != '' and not misc.execute('losetup', '-d', dev):
+            if (dev != '' and dev != 'unused' and
+                not misc.execute('losetup', '-d', dev)):
                 raise InstallStepError(
                     "Failed to detach loopback device %s" % dev)
 
@@ -1014,7 +1162,7 @@ exit 0"""
                 assert cache._depcache.BrokenCount == 0
 
 
-    def install_language_packs(self):
+    def select_language_packs(self):
         langpacks = []
         try:
             langpack_db = self.db.get('pkgsel/language-packs')
@@ -1053,7 +1201,33 @@ exit 0"""
             # More extensive language support packages.
             to_install.append('language-support-%s' % lp)
 
-        self.do_install(to_install, langpacks=True)
+        self.record_installed(to_install)
+        self.langpacks = to_install
+
+    def install_language_packs(self):
+        self.do_install(self.langpacks)
+
+        cache = Cache()
+        incomplete = False
+        for pkg in self.langpacks:
+            cachedpkg = self.get_cache_pkg(cache, pkg)
+            if cachedpkg is None or not cachedpkg.isInstalled:
+                incomplete = True
+                break
+        if incomplete:
+            language_support_dir = \
+                os.path.join(self.target, 'usr/share/language-support')
+            update_notifier_dir = \
+                os.path.join(self.target, 'var/lib/update-notifier/user.d')
+            for note in ('incomplete-language-support-gnome.note',
+                         'incomplete-language-support-qt.note'):
+                notepath = os.path.join(language_support_dir, note)
+                if os.path.exists(notepath):
+                    if not os.path.exists(update_notifier_dir):
+                        os.makedirs(update_notifier_dir)
+                    shutil.copy(notepath,
+                                os.path.join(update_notifier_dir, note))
+                    break
 
 
     def configure_timezone(self):
@@ -1109,6 +1283,8 @@ exit 0"""
             words = line.split()
             if words[1] != 'partition':
                 continue
+            if words[0].startswith('/dev/ramzswap'):
+                continue
             size = int(words[2])
             if size > biggest_size:
                 biggest_size = size
@@ -1213,6 +1389,7 @@ exit 0"""
         packages = ['linux-image-' + self.kernel_version,
                     'linux-restricted-modules-' + self.kernel_version,
                     'usplash',
+                    'splashy',
                     'popularity-contest',
                     'libpaper1',
                     'ssl-cert']
@@ -1413,7 +1590,7 @@ exit 0"""
             subarch = archdetect.communicate()[0].strip()
 
             try:
-                if subarch.startswith('amd64/') or subarch.startswith('i386/'):
+                if subarch.startswith('amd64/') or subarch.startswith('i386/') or subarch.startswith('lpia/'):
                     from ubiquity.components import grubinstaller
                     dbfilter = grubinstaller.GrubInstaller(None)
                     ret = dbfilter.run_command(auto_process=True)
@@ -1454,18 +1631,13 @@ exit 0"""
                 continue
         return brokenpkgs
 
-    def do_install(self, to_install, langpacks=False):
-        if langpacks:
+    def do_install(self, to_install):
+        if self.langpacks:
             self.db.progress('START', 0, 10, 'ubiquity/langpacks/title')
         else:
             self.db.progress('START', 0, 10, 'ubiquity/install/title')
         self.db.progress('INFO', 'ubiquity/install/find_installables')
 
-        if langpacks:
-            # ... otherwise we're installing packages that have already been
-            # recorded
-            self.record_installed(to_install)
-
         self.db.progress('REGION', 0, 1)
         fetchprogress = DebconfFetchProgress(
             self.db, 'ubiquity/install/title',
@@ -1485,7 +1657,7 @@ exit 0"""
 
         self.db.progress('SET', 1)
         self.db.progress('REGION', 1, 10)
-        if langpacks:
+        if self.langpacks:
             fetchprogress = DebconfFetchProgress(
                 self.db, 'ubiquity/langpacks/title', None,
                 'ubiquity/langpacks/packages')
@@ -1541,23 +1713,9 @@ exit 0"""
         self.db.progress('STOP')
 
 
-    def do_remove(self, to_remove, recursive=False):
-        self.db.progress('START', 0, 5, 'ubiquity/install/title')
-        self.db.progress('INFO', 'ubiquity/install/find_removables')
-
-        fetchprogress = DebconfFetchProgress(
-            self.db, 'ubiquity/install/title',
-            'ubiquity/install/apt_indices_starting',
-            'ubiquity/install/apt_indices')
-        cache = Cache()
-
-        if cache._depcache.BrokenCount > 0:
-            syslog.syslog(
-                'not processing removals, since there are broken packages: '
-                '%s' % ', '.join(self.broken_packages(cache)))
-            self.db.progress('STOP')
-            return
-
+    def get_remove_list(self, cache, to_remove, recursive=False):
+        to_remove = set(to_remove)
+        all_removed = set()
         while True:
             removed = set()
             for pkg in to_remove:
@@ -1609,6 +1767,28 @@ exit 0"""
             if not removed:
                 break
             to_remove -= removed
+            all_removed |= removed
+        return all_removed
+
+
+    def do_remove(self, to_remove, recursive=False):
+        self.db.progress('START', 0, 5, 'ubiquity/install/title')
+        self.db.progress('INFO', 'ubiquity/install/find_removables')
+
+        fetchprogress = DebconfFetchProgress(
+            self.db, 'ubiquity/install/title',
+            'ubiquity/install/apt_indices_starting',
+            'ubiquity/install/apt_indices')
+        cache = Cache()
+
+        if cache._depcache.BrokenCount > 0:
+            syslog.syslog(
+                'not processing removals, since there are broken packages: '
+                '%s' % ', '.join(self.broken_packages(cache)))
+            self.db.progress('STOP')
+            return
+
+        self.get_remove_list(cache, to_remove, recursive)
 
         self.db.progress('SET', 1)
         self.db.progress('REGION', 1, 5)
@@ -1749,7 +1929,7 @@ exit 0"""
                                    os.path.join('/etc/gdm', name));
                         break
 
-                kdmrc = os.path.join(self.target, 'etc/kde3/kdm/kdmrc')
+                kdmrc = os.path.join(self.target, 'etc/kde4/kdm/kdmrc')
                 if os.path.isfile(kdmrc):
                     misc.execute('sed', '-i.oem', '-r',
                                  '-e', 's/^#?AutoLoginEnable=.*$/AutoLoginEnable=true/',
@@ -1757,15 +1937,6 @@ exit 0"""
                                  '-e', 's/^#?AutoReLogin=.*$/AutoReLogin=true/',
                                  kdmrc)
 
-                if osextras.find_on_path_root(self.target, 'kpersonalizer'):
-                    kpersonalizerrc = os.path.join(self.target,
-                                                   'etc/kde3/kpersonalizerrc')
-                    if not os.path.isfile(kpersonalizerrc):
-                        kpersonalizerrc_file = open(kpersonalizerrc, 'w')
-                        print >>kpersonalizerrc_file, '[General]'
-                        print >>kpersonalizerrc_file, 'FirstLogin=false'
-                        kpersonalizerrc_file.close()
-                        open('%s.created-by-oem', 'w').close()
 		# Carry the locale setting over to the installed system.
 		# This mimics the behavior in 01oem-config-udeb.
                 di_locale = self.db.get('debian-installer/locale')
@@ -1831,7 +2002,35 @@ exit 0"""
         # them automatically here.
         self.do_remove(difference)
 
-
+    def remove_broken_cdrom(self):
+        fstab = os.path.join(self.target, 'etc/fstab')
+        ret = []
+        try:
+            fp = open(fstab)
+            for line in fp:
+                l = line.split()
+                if len(l) > 2:
+                    if l[1].startswith('/cdrom') or l[1].startswith('/media/cdrom'):
+                        try:
+                            fstype = subprocess.Popen(
+                                ['vol_id', '--type', l[0]],
+                                stdout=subprocess.PIPE).communicate()[0].rstrip('\n')
+                            if fstype != 'iso9660' and fstype != 'udf':
+                                continue
+                        except OSError:
+                            pass
+                ret.append(line)
+            fp.close()
+            fp = open(fstab, 'w')
+            fp.writelines(ret)
+        except Exception, e:
+            syslog.syslog(syslog.LOG_ERR, 'Exception during installation:')
+            syslog.syslog(syslog.LOG_ERR,
+                'Unable to process /etc/fstab: ' + str(e))
+        finally:
+            if fp:
+                fp.close()
+            
     def cleanup(self):
         """Miscellaneous cleanup tasks."""
 
