@@ -157,7 +157,7 @@ class Partman(FilteredCommand):
                        want_script, want_arg=None):
         (script, arg, option) = self.must_find_one_script(
             question, menu_options, want_script, want_arg)
-        self.preseed(question, option)
+        self.preseed(question, option, seen=False)
 
     def split_devpart(self, devpart):
         dev, part_id = devpart.split('//', 1)
@@ -220,6 +220,9 @@ class Partman(FilteredCommand):
             elif method == 'dont_use':
                 question = 'partman-basicmethods/text/dont_use'
                 yield (method, 'dontuse', self.description(question))
+            elif method == 'efi':
+                if os.path.exists('/var/lib/partman/efi'):
+                    yield (method, method, self.method_description(method))
             else:
                 yield (method, method, self.method_description(method))
 
@@ -284,42 +287,22 @@ class Partman(FilteredCommand):
             self.debug('Partman: update_partitions = %s',
                        self.update_partitions)
 
-    def parse_size(self, size_str):
-        (num, unit) = size_str.split(' ', 1)
-        if ',' in num:
-            (size_int, size_frac) = num.split(',', 1)
-        else:
-            (size_int, size_frac) = num.split('.', 1)
-        size = float(str("%s.%s" % (size_int, size_frac)))
-        # partman measures sizes in decimal units
-        if unit == 'B':
-            pass
-        elif unit == 'kB':
-            size *= 1000
-        elif unit == 'MB':
-            size *= 1000000
-        elif unit == 'GB':
-            size *= 1000000000
-        elif unit == 'TB':
-            size *= 1000000000000
-        return size
-
     def subst(self, question, key, value):
         if question == 'partman-partitioning/new_size':
             if self.building_cache and self.autopartition_question is None:
                 state = self.__state[-1]
                 assert state[0] == 'partman/active_partition'
                 partition = self.partition_cache[state[1]]
-                if key == 'MINSIZE':
-                    partition['resize_min_size'] = self.parse_size(value)
-                elif key == 'MAXSIZE':
-                    partition['resize_max_size'] = self.parse_size(value)
-            if key == 'MINSIZE':
-                self.resize_min_size = self.parse_size(value)
-            elif key == 'MAXSIZE':
-                self.resize_max_size = self.parse_size(value)
+                if key == 'RAWMINSIZE':
+                    partition['resize_min_size'] = int(value)
+                elif key == 'RAWMAXSIZE':
+                    partition['resize_max_size'] = int(value)
+            if key == 'RAWMINSIZE':
+                self.resize_min_size = int(value)
+            elif key == 'RAWMAXSIZE':
+                self.resize_max_size = int(value)
             elif key == 'ORISIZE':
-                self.resize_orig_size = self.parse_size(value)
+                self.resize_orig_size = int(value)
             elif key == 'PATH':
                 self.resize_path = value
 
@@ -391,7 +374,7 @@ class Partman(FilteredCommand):
                 # it doesn't expand variables in the result of METAGET
                 # choices-c. All locales have the same variables anyway so
                 # it doesn't matter.
-                self.preseed(question, self.auto_state[1])
+                self.preseed(question, self.auto_state[1], seen=False)
                 self.succeeded = True
                 return True
             else:
@@ -402,6 +385,29 @@ class Partman(FilteredCommand):
                     del choices[choices.index(self.resize_desc)]
                 except ValueError:
                     pass
+            regain_privileges()
+            # {'/dev/sda' : [ ('/dev/sda1', 24973243297), ('free', 23492732) ], '/dev/sdb' : .., }
+            parted = parted_server.PartedServer()
+            layout = {}
+            for disk in parted.disks():
+                parted.select_disk(disk)
+                ret = []
+                total = 0
+                for partition in parted.partitions():
+                    size = int(partition[2])
+                    total = total + size
+                for partition in parted.partitions():
+                    print 'partition: %s' % str(partition)
+                    size = int(partition[2])
+                    if partition[4] == 'free':
+                        dev = 'free'
+                    else:
+                        dev = partition[5]
+                    ret.append((dev, size / float(total)))
+                layout[disk] = ret
+
+            self.frontend.set_disk_layout(layout)
+            drop_privileges()
             self.frontend.set_autopartition_choices(
                 choices, self.extra_options,
                 self.resize_desc, self.manual_desc)
@@ -414,7 +420,7 @@ class Partman(FilteredCommand):
                 return False
             else:
                 assert self.extra_choice is not None
-                self.preseed(question, self.extra_choice)
+                self.preseed(question, self.extra_choice, seen=False)
                 self.succeeded = True
                 return True
 
@@ -454,7 +460,8 @@ class Partman(FilteredCommand):
                         partition = self.partition_cache[state[1]]
                         self.debug('Partman: Building cache (%s)',
                                    partition['parted']['path'])
-                        self.preseed(question, partition['display'])
+                        self.preseed(question, partition['display'],
+                                     seen=False)
                         return True
                     else:
                         # Finished building the cache.
@@ -502,7 +509,8 @@ class Partman(FilteredCommand):
                             if rebuild_all or arg not in self.partition_cache:
                                 self.partition_cache[arg] = {
                                     'dev': dev,
-                                    'id': part_id
+                                    'id': part_id,
+                                    'parent': dev.replace('=', '/')
                                 }
                         else:
                             if rebuild_all or arg not in self.disk_cache:
@@ -577,7 +585,8 @@ class Partman(FilteredCommand):
                         self.debug('Partman: Building cache (%s)',
                                    partition['parted']['path'])
                         self.__state.append([question, devpart, None])
-                        self.preseed(question, partition['display'])
+                        self.preseed(question, partition['display'],
+                                     seen=False)
                         return True
                     else:
                         self.debug('Partman: Finished building cache '
@@ -637,7 +646,7 @@ class Partman(FilteredCommand):
                 if devpart in self.disk_cache:
                     disk = self.disk_cache[devpart]
                     # No need to use self.__state to keep track of this.
-                    self.preseed(question, disk['display'])
+                    self.preseed(question, disk['display'], seen=False)
                 return True
 
             elif self.creating_partition:
@@ -645,7 +654,7 @@ class Partman(FilteredCommand):
                 if devpart in self.partition_cache:
                     partition = self.partition_cache[devpart]
                     self.__state.append([question, devpart, None])
-                    self.preseed(question, partition['display'])
+                    self.preseed(question, partition['display'], seen=False)
                 return True
 
             elif self.editing_partition:
@@ -653,7 +662,7 @@ class Partman(FilteredCommand):
                 if devpart in self.partition_cache:
                     partition = self.partition_cache[devpart]
                     self.__state.append([question, devpart, None])
-                    self.preseed(question, partition['display'])
+                    self.preseed(question, partition['display'], seen=False)
                 return True
 
             elif self.deleting_partition:
@@ -661,7 +670,7 @@ class Partman(FilteredCommand):
                 if devpart in self.partition_cache:
                     partition = self.partition_cache[devpart]
                     # No need to use self.__state to keep track of this.
-                    self.preseed(question, partition['display'])
+                    self.preseed(question, partition['display'], seen=False)
                 return True
 
             elif self.undoing:
@@ -679,9 +688,9 @@ class Partman(FilteredCommand):
                     self.extended_description(question),
                     ('ubiquity/text/go_back', 'ubiquity/text/continue'))
                 if response is None or response == 'ubiquity/text/continue':
-                    self.preseed(question, 'true')
+                    self.preseed(question, 'true', seen=False)
                 else:
-                    self.preseed(question, 'false')
+                    self.preseed(question, 'false', seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -711,7 +720,7 @@ class Partman(FilteredCommand):
                 if re.search(r'^[0-9.]+$', size):
                     # ensure megabytes just in case partman's semantics change
                     size += 'M'
-                self.preseed(question, size)
+                self.preseed(question, size, seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -719,9 +728,9 @@ class Partman(FilteredCommand):
         elif question == 'partman-partitioning/new_partition_type':
             if self.creating_partition:
                 if self.creating_partition['type'] == PARTITION_TYPE_PRIMARY:
-                    self.preseed(question, 'Primary')
+                    self.preseed(question, 'Primary', seen=False)
                 else:
-                    self.preseed(question, 'Logical')
+                    self.preseed(question, 'Logical', seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -730,9 +739,9 @@ class Partman(FilteredCommand):
             if self.creating_partition:
                 if (self.creating_partition['place'] ==
                     PARTITION_PLACE_BEGINNING):
-                    self.preseed(question, 'Beginning')
+                    self.preseed(question, 'Beginning', seen=False)
                 else:
-                    self.preseed(question, 'End')
+                    self.preseed(question, 'End', seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -747,7 +756,7 @@ class Partman(FilteredCommand):
                     if state[2] < len(partition['active_partition_build']):
                         # Move on to the next item.
                         visit = partition['active_partition_build']
-                        self.preseed(question, visit[state[2]][2])
+                        self.preseed(question, visit[state[2]][2], seen=False)
                         return True
                     else:
                         # Finished building the cache for this submenu; go
@@ -786,7 +795,7 @@ class Partman(FilteredCommand):
                 if visit:
                     partition['active_partition_build'] = visit
                     self.__state.append([question, state[1], 0])
-                    self.preseed(question, visit[0][2])
+                    self.preseed(question, visit[0][2], seen=False)
                     return True
                 else:
                     # Back up to the previous menu.
@@ -821,7 +830,7 @@ class Partman(FilteredCommand):
                     item = visit[state[2]]
                     scripts = self.find_script(menu_options, None, item)
                     if scripts:
-                        self.preseed(question, scripts[0][2])
+                        self.preseed(question, scripts[0][2], seen=False)
                         return True
                     state[2] += 1
 
@@ -848,7 +857,7 @@ class Partman(FilteredCommand):
                 if self.auto_state is not None:
                     # Proceed through confirmation question; we'll back up
                     # later.
-                    self.preseed(question, 'true')
+                    self.preseed(question, 'true', seen=False)
                     return True
                 else:
                     response = self.frontend.question_dialog(
@@ -857,16 +866,16 @@ class Partman(FilteredCommand):
                         ('ubiquity/text/go_back', 'ubiquity/text/continue'))
                     if (response is None or
                         response == 'ubiquity/text/continue'):
-                        self.preseed(question, 'true')
+                        self.preseed(question, 'true', seen=False)
                     else:
-                        self.preseed(question, 'false')
+                        self.preseed(question, 'false', seen=False)
                     return True
             elif self.building_cache:
                 state = self.__state[-1]
                 assert state[0] == 'partman/active_partition'
                 # Proceed through to asking for the size; don't worry, we'll
                 # back up from there.
-                self.preseed(question, 'true')
+                self.preseed(question, 'true', seen=False)
                 return True
             elif self.editing_partition:
                 response = self.frontend.question_dialog(
@@ -874,9 +883,9 @@ class Partman(FilteredCommand):
                     self.extended_description(question),
                     ('ubiquity/text/go_back', 'ubiquity/text/continue'))
                 if response is None or response == 'ubiquity/text/continue':
-                    self.preseed(question, 'true')
+                    self.preseed(question, 'true', seen=False)
                 else:
-                    self.preseed(question, 'false')
+                    self.preseed(question, 'false', seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -895,7 +904,7 @@ class Partman(FilteredCommand):
                     if self.bad_auto_size:
                         self.bad_auto_size = False
                         return False
-                    self.preseed(question, '%d%%' % self.extra_choice)
+                    self.preseed(question, self.extra_choice, seen=False)
                     self.succeeded = True
                     return True
             elif self.building_cache:
@@ -909,7 +918,7 @@ class Partman(FilteredCommand):
                 if re.search(r'^[0-9.]+$', size):
                     # ensure megabytes just in case partman's semantics change
                     size += 'M'
-                self.preseed(question, size)
+                self.preseed(question, size, seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -963,9 +972,9 @@ class Partman(FilteredCommand):
                 mountpoint = request['mountpoint']
 
                 if mountpoint == '' or mountpoint is None:
-                    self.preseed(question, 'Do not mount it')
+                    self.preseed(question, 'Do not mount it', seen=False)
                 else:
-                    self.preseed(question, 'Enter manually')
+                    self.preseed(question, 'Enter manually', seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -979,7 +988,7 @@ class Partman(FilteredCommand):
                 if 'bad_mountpoint' in request:
                     return False
 
-                self.preseed(question, request['mountpoint'])
+                self.preseed(question, request['mountpoint'], seen=False)
                 return True
             else:
                 raise AssertionError, "Arrived at %s unexpectedly" % question
@@ -989,7 +998,7 @@ class Partman(FilteredCommand):
                 self.db.set('ubiquity/partman-made-changes', 'true')
             else:
                 self.db.set('ubiquity/partman-made-changes', 'false')
-            self.preseed(question, 'true')
+            self.preseed(question, 'true', seen=False)
             self.succeeded = True
             self.done = True
             return True
@@ -1000,9 +1009,9 @@ class Partman(FilteredCommand):
                     self.description(question),
                     self.extended_description(question),
                     self.choices(question), use_templates=False)
-                self.preseed(question, response)
+                self.preseed(question, response, seen=False)
             else:
-                self.preseed(question, 'unhandled')
+                self.preseed(question, 'unhandled', seen=False)
             return True
 
         elif question == 'partman/exception_handler_note':
@@ -1028,9 +1037,9 @@ class Partman(FilteredCommand):
             else:
                 answer = not answer_reversed
             if answer:
-                self.preseed(question, 'true')
+                self.preseed(question, 'true', seen=False)
             else:
-                self.preseed(question, 'false')
+                self.preseed(question, 'false', seen=False)
             return True
 
         return FilteredCommand.run(self, priority, question)
