@@ -21,9 +21,10 @@ from ubiquity.filteredcommand import FilteredCommand
 
 class SiloInstaller(FilteredCommand):
     def prepare(self):
-        return (['/usr/lib/ubiquity/silo-installer/silo-installer'], ['ERROR'])
+        return (['/usr/lib/ubiquity/silo-installer/silo-installer'], ['ERROR'],
+                {'PATH': '/usr/lib/ubiquity/silo-installer:' + os.environ['PATH']})
 
     def error(self, priority, question):
         self.frontend.error_dialog(self.description(question),
                                    self.extended_description(question))
-        return super(SiloInstaller, self).error(priority, question)
+        return FilteredCommand.error(self, priority, question)
