@@ -18,21 +18,43 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 from ubiquity.filteredcommand import FilteredCommand
+import debconf
 
 class UserSetup(FilteredCommand):
     def prepare(self):
+        if self.frontend.get_hostname() == '':
+            try:
+                hostname = self.db.get('netcfg/get_hostname')
+                domain = self.db.get('netcfg/get_domain')
+                if hostname and domain:
+                    hostname = '%s.%s' % (hostname, domain)
+                if hostname != '':
+                    self.frontend.set_hostname(hostname)
+            except debconf.DebconfError:
+                pass
         if self.frontend.get_fullname() == '':
-            fullname = self.db.get('passwd/user-fullname')
-            if fullname != '':
-                self.frontend.set_fullname(fullname)
+            try:
+                fullname = self.db.get('passwd/user-fullname')
+                if fullname != '':
+                    self.frontend.set_fullname(fullname)
+            except debconf.DebconfError:
+                pass
         if self.frontend.get_username() == '':
-            username = self.db.get('passwd/username')
-            if username != '':
-                self.frontend.set_username(username)
+            try:
+                username = self.db.get('passwd/username')
+                if username != '':
+                    self.frontend.set_username(username)
+            except debconf.DebconfError:
+                pass
+        try:
+            auto_login = self.db.get('passwd/auto-login')
+            self.frontend.set_auto_login(auto_login == 'true')
+        except debconf.DebconfError:
+            pass
 
         questions = ['^passwd/user-fullname$', '^passwd/username$',
                      '^passwd/user-password$', '^passwd/user-password-again$',
-                     'ERROR']
+                     '^passwd/auto-login$', 'ERROR']
         return (['/usr/lib/ubiquity/user-setup/user-setup-ask', '/target'],
                 questions)
 
@@ -46,6 +68,7 @@ class UserSetup(FilteredCommand):
         username = self.frontend.get_username()
         password = self.frontend.get_password()
         password_confirm = self.frontend.get_verified_password()
+        auto_login = self.frontend.get_auto_login()
 
         self.preseed('passwd/user-fullname', fullname)
         self.preseed('passwd/username', username)
@@ -56,6 +79,16 @@ class UserSetup(FilteredCommand):
             self.preseed('passwd/user-uid', '29999')
         else:
             self.preseed('passwd/user-uid', '')
+        self.preseed_bool('passwd/auto-login', auto_login)
+        
+        hostname = self.frontend.get_hostname()
+        if hostname is not None and hostname != '':
+            hd = hostname.split('.', 1)
+            self.preseed('netcfg/get_hostname', hd[0])
+            if len(hd) > 1:
+                self.preseed('netcfg/get_domain', hd[1])
+            else:
+                self.preseed('netcfg/get_domain', '')
 
         FilteredCommand.ok_handler(self)
 
